***************************************************************************************************
*** news_type_20160710.do 
*** 07/10/2016
*** News Shocks in Open Economies: Evidence from Giant Oil Discoveries
***            Arezki, Ramey and Sheng (2015)
*** Description:
***     This program presents the code for 
***     the dynamic effect of two types of discoveries on macro variables.
***     Onshore discoveries v.s. offshore discoveries
***
*** Requires:
***     newspanel2016.dta
***     graph_forQJE.do
*** Note: You need to run the code below twice 
*** to save the IRFs for two types of discoveries separately
*** by choosing the type of discovery in line 163-168,
*** and then use graph_forQJE.do to plot Figure VIII
***************************************************************************************************

version 12

clear all
drop _all
graph drop _all

set more off
set matsize 11000
capture log close


** cd to the current folder


** create the folder to save results 
global savefolder twotype
shell mkdir $savefolder

log using $savefolder\news_twotype.smcl, replace




************************************************************************************************************
use newspanel2016.dta,clear
disp "DateTime: $S_DATE $S_TIME"

xtset  ifscode year
xi I.year, prefix(_I)


************************************************************************************************************ 
** Part I: Prepare variables for regression
************************************************************************************************************ 

** Benchmark measure of size

gen size=sizerealistic
label var size "NPV with risk adjusted discount rate and realistic production profile"


** Generate sufficient lags for measures of discovery shocks
local discvar size sizeonshore sizeoffshore 

foreach var of varlist `discvar' {
  forvalues i=1/20 {
  g `var'`i'=L`i'.`var'
  }
}



** Generate lagged dependent variables

local depvar  ca saving inv lngdplcu lnclcu emprate  

foreach var of varlist `depvar' {
gen `var'1=L.`var'
gen `var'2=L2.`var'
gen `var'3=L3.`var'
gen `var'4=L4.`var'
}


** Sample selection 

*** BENCHMARK SETTING: all countries during 1980-2012 
drop if year<1980 | year>2012


** Generate country specific quadratic trends
egen tt=group(year)
gen ttsquare=tt^2

egen float dum = group(ifscode) 
sum dum
local dummax=r(max)
xi I.dum , prefix(cc_)
*g constant=1
g tt1=tt
g ttsquare1=ttsquare
  forvalues i=2/`dummax' {
  g tt`i'=tt*cc_dum_`i'
  g ttsquare`i'=ttsquare*cc_dum_`i'
  }
  
drop tt ttsquare


*******************************************************************************
* Dynamic panel with distributed lag model: 
* AR(p) with exogenous X and its lags (q)
* You can choose p=1 or p=2
* h-1 is the irf horitonal periods
*******************************************************************************

** Baseline setting
local p=1
local q =10
local h=20+1

local ci=90  // confidence interval

local cia=68 // alternative CI





** Key dependent variables inlcudes: ca saving inv lngdplcu lnclcu emprate 

 global keydep ca saving inv lngdplcu lnclcu emprate 

 
******************************************************************************************************************
* Main code to compute IRFs from the dynamic panel with distributed lag model
******************************************************************************************************************

foreach var of varlist $keydep {
gen start=1

if `var'==ca | `var'==saving | `var'==inv | `var'==invprivgdp | `var'==invpubgdp | `var'==govspgdp  | `var'==invpubgdp  {

	** NPV with its lags: for ratio variables
 
	** two types of discoveries: onshore vs offshore
	 global xlist sizeonshore sizeonshore1-sizeonshore`q'  sizeoffshore sizeoffshore1-sizeoffshore`q' 


 }

else {

 ** Add control for country specific second order polynomial trend for log level variables
  
	** two types of discoveries: onshore vs offshore
	global xlist sizeonshore sizeonshore1-sizeonshore`q'  sizeoffshore sizeoffshore1-sizeoffshore`q' tt*
    

 }


 ************************************************
 ** Compute IRF for offshore discoveries
  * local indep sizeonshore
 
 ** Compute IRF for offshore discoveries
   local indep sizeoffshore
 ************************************************
 
 
 ** global dependent and independent variables.
	global dep `var'
	global lagdep `var'1-`var'`p'

 ** global yearly fixed effect
    global yearlist _Iyear_1980-_Iyear_2012



*** Driscoll-Kraay Estimation************************************

xtscc $dep $lagdep  $xlist $yearlist , fe 


** Computing IRF and hypothesis testing 

** USE nlcom and recursive macro subsitutions 

if `p'==1 { 
local rho1 "_b[`var'1]"
local irf0 "_b[`indep']"


forvalues i = 1/`q' {
  local i1 = `i'-1
  local irf`i' "(`rho1'*`irf`i1'' + _b[`indep'`i'])" 	
  
}

local qnx=`q'+1

forvalues i = `qnx'/20 {
 local i1 = `i'-1
 local irf`i' "(`rho1'*`irf`i1'')"
}
** IRF: use the following code with a factor of 1000 otherwise it may show the error message "Maximum number of iterations exceeded."
nlcom (irf0:1000*`irf0') (irf1:1000*`irf1') (irf2:1000*`irf2') (irf3:1000*`irf3') (irf4:1000*`irf4') (irf5:1000*`irf5') (irf6:1000*`irf6') (irf7:1000*`irf7')  ///
(irf8:1000*`irf8') (irf9:1000*`irf9') (irf10:1000*`irf10') (irf11:1000*`irf11') (irf12:1000*`irf12') (irf13:1000*`irf13') (irf14:1000*`irf14') (irf15:1000*`irf15')  ///
 (irf16:1000*`irf16') (irf17:1000*`irf17') (irf18:1000*`irf18') (irf19:1000*`irf19') (irf20:1000*`irf20'), level(`ci')  post

  nlcom (irf0:_b[irf0]/1000) (irf1:_b[irf1]/1000) (irf2:_b[irf2]/1000) (irf3:_b[irf3]/1000) (irf4:_b[irf4]/1000) (irf5:_b[irf5]/1000) (irf6:_b[irf6]/1000) (irf7:_b[irf7]/1000)  ///
(irf8:_b[irf8]/1000) (irf9:_b[irf9]/1000) (irf10:_b[irf10]/1000) (irf11:_b[irf11]/1000) (irf12:_b[irf12]/1000) (irf13:_b[irf13]/1000) (irf14:_b[irf14]/1000) (irf15:_b[irf15]/1000)  ///
 (irf16:_b[irf16]/1000) (irf17:_b[irf17]/1000) (irf18:_b[irf18]/1000) (irf19:_b[irf19]/1000) (irf20:_b[irf20]/1000), level(`ci')  post


*** save all IRF in a separate data
parmest, list(,) saving($savefolder\irfnl_`var'_`indep'.dta, replace) level(`ci')

}



else{
di "This code only provides solutions to order `p'=1 for `q'<20" 
}


** save impulse response.
preserve

use $savefolder\irfnl_`var'_`indep'.dta, clear
gen n=_n-1
sort n
label var n "Year"
local tail=(100-`cia')/200
gen max`cia'=estimate+invttail(dof,`tail')*std
gen min`cia'=estimate-invttail(dof,`tail')*std

*twoway rarea min`ci' max`ci' n, color(gs14)|| connect estimate n, lcolor(navy ..) legend(off) name(irfnl_`var',replace)
*twoway rarea min`ci' max`ci' n, color(gs14) || rarea min`cia' max`cia' n,  color(gs13) || connect estimate n, color(blue) legend(off) name(irfnl_`var',replace)
*graph save $savefolder\irfnl_`var'_`indep'.gph, replace
save $savefolder\irfnl_`var'_`indep'.dta, replace
restore

gen end=1
drop start-end

}





**********************************************************************
* Merge all IRF


foreach file in "lngdplcu" "ca" "saving" "inv" "lnclcu" "emprate" {
use $savefolder\irfnl_`file'_`indep'.dta, clear
ren n period
move period parm
label var estimate ""
ren estimate `file'
ren min90 `file'_min90
ren max90 `file'_max90
ren min68 `file'_min68
ren max68 `file'_max68

drop parm stderr dof t p 
save $savefolder\irf_temp_`file'_`indep'.dta, replace
}


use $savefolder\irf_temp_lngdplcu_`indep'.dta,clear
foreach file in "ca" "saving" "inv" "lnclcu" "emprate"  {
merge 1:1 period using $savefolder\irf_temp_`file'_`indep'.dta
drop _merge
}

sort period

ren period year
save $savefolder\irf_all_`indep'.dta, replace


**Use graph_forQJE.do file to produce Figures in the paper. 




** Erase all intermediate data files.
foreach file in "lngdplcu" "ca" "saving" "inv" "lnclcu" "emprate"  {
erase $savefolder\irf_temp_`file'_`indep'.dta
erase $savefolder\irfnl_`file'_`indep'.dta
}

log close


